/*
 * File: edu/cs/rochester/WordNet/browser/BrowserOSX.java
 * Creator: George Ferguson
 * Created: Mon Jun 14 15:43:14 2010
 * Time-stamp: <Tue Jun 15 21:05:27 EDT 2010 ferguson>
 *
 * Copyright 2010 George Ferguson, ferguson@cs.rochester.edu.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.rochester.cs.WordNet.browser;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.apple.eawt.*;

/**
 * Platform-specific Browser extensions for OSX.
 * @see http://developer.apple.com/mac/library/documentation/Java/Conceptual/Java14Development/03-JavaDeployment/JavaDeployment.html
 * @see http://developer.apple.com/mac/library/documentation/Java/Reference/Java_InfoplistRef/Articles/JavaDictionaryInfo.plistKeys.html
 * @see http://developer.apple.com/Mac/library/documentation/Java/Reference/Java_PropertiesRef/Articles/JavaSystemProperties.html
 * @see http://developer.apple.com/mac/library/documentation/Java/Reference/1.5.0/appledoc/api/com/apple/eawt/package-summary.html
 */
public class BrowserOSX extends Browser {

    /**
     * Entry point for the BrowserOSX class.
     * Performs various platform-specific initializations, then
     * do what Browser.main() does to start the app.
     */
    public static void main(String[] argv) {
	initSystemProperties();
	// Off we go...
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		new BrowserOSX().setVisible(true);
	    }});
    }

    /**
     * Setup various system properties that make the Java app more like
     * a Mac OSX app.
     * Some of these can also be done in the Info.plist used to launch
     * the app.
     */
    protected static void initSystemProperties() {
	try {
	    // take the menu bar off the jframe
	    System.setProperty("apple.laf.useScreenMenuBar", "true");
	    // set the name of the application menu item
	    System.setProperty("com.apple.mrj.application.apple.menu.about.name", APP_TITLE);
	    // set the look and feel
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception ex) {
	    System.err.println(ex.getMessage());
	}
    }

    /**
     * Browser method to create and return this Browser's File menu.
     * In OSX, several items are on the application menu rather than
     * the File menu.
     */
    protected JMenu createFileMenu() {
	initApplicationMenu();
	JMenu menu = new JMenu("File");
	menu.add(createSaveAsMenuItem());
	menu.addSeparator();
	menu.add(createPageSetupMenuItem());
	menu.add(createPrintMenuItem());
	return menu;
    }

    /**
     * Browser method to create and return this Browser's Edit menu.
     * In OSX, there's no Delete item on this menu, and the Preferences
     * item is on the application menu.
     */
    protected JMenu createEditMenu() {
	JMenu menu = super.createEditMenu();
	for (int i=0; i < menu.getItemCount(); i++) {
	    JMenuItem item = menu.getItem(i);
	    if (item != null) {
		String label = item.getText();
		if (label.equals("Delete")) {
		    menu.remove(item);
		    i -= 1;
		} else if (label.startsWith("Preferences")) {
		    menu.remove(i-1); // Separator
		    menu.remove(item);
		    i -= 2;
		}
	    }
	}
	return menu;
    }

    /**
     * Browser method to create and return this Browser's Help menu.
     * In OSX, the About item is on the application menu, and the Help
     * menu invokes Apple's Help Viewer.
     * @see http://developer.apple.com/mac/library/documentation/Java/Reference/1.5.0/appledoc/api/com/apple/eawt/Application.html#openHelpViewer()
     * @see http://developer.apple.com/qa/qa2001/qa1022.html
     * @see http://developer.apple.com/mac/library/documentation/Carbon/Conceptual/ProvidingUserAssitAppleHelp/authoring_help/authoring_help_book.html
     */
    protected JMenu createHelpMenu() {
	JMenu menu = new JMenu("Help");
	JMenuItem item = new JMenuItem(APP_TITLE + " Help");
	item.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Application.getApplication().openHelpViewer();
		}
	    });
	menu.add(item);
	return menu;
    }

    /**
     * Browser method to set the given key as the mnemonic for the
     * given item (Menu or MenuItem).
     * Use of mnemonic keys in menus is discouraged in OSX, so this method
     * is overridden to do nothing.
     */
    protected void setMnemonic(AbstractButton item, int key, int index) {
    }

    /**
     * Initialize this Browser's OSX application menu.
     * This is an Apple eawt extension.
     */
    protected void initApplicationMenu() {
	Application app = Application.getApplication();
	app.setEnabledPreferencesMenu(true);
	app.addApplicationListener(new ApplicationAdapter() {
		public void handleAbout(ApplicationEvent e) {
		    e.setHandled(true);
		    showAbout();
		}
		public void handlePreferences(ApplicationEvent e) {
		    showPreferences();
		}
		public void handlePrintFile(ApplicationEvent e) {
		}
		public void handleQuit(ApplicationEvent e) {
		    quit();
		}
	    });
    }

}
